@@ -18,6 +18,7 @@ static int compare_pages(const cb_document_page_meta_t* page1, const cb_document
 static bool read_archive(cb_document_t* cb_document, const char* archive, girara_list_t* supported_extensions);
 static char* get_extension(const char* path);
 static void cb_document_page_meta_free(cb_document_page_meta_t* meta);
+static bool read_dir(cb_document_t* cb_document, const char* directory, girara_list_t* supported_extensions);
 
 zathura_error_t
 cb_document_open(zathura_document_t* document)
@@ -25,7 +26,6 @@ cb_document_open(zathura_document_t* document)
   if (document == NULL) {
     return ZATHURA_ERROR_INVALID_ARGUMENTS;
   }
-
   cb_document_t* cb_document = g_malloc0(sizeof(cb_document_t));
 
   /* archive path */
@@ -58,8 +58,14 @@ cb_document_open(zathura_document_t* document)
   }
 
   /* read files recursively */
-  if (read_archive(cb_document, path, supported_extensions) == false) {
-    goto error_free;
+  if (g_file_test(path, G_FILE_TEST_IS_DIR)) {
+    if (read_dir(cb_document, path, supported_extensions) == false) {
+      goto error_free;
+    }
+  } else {
+    if (read_archive(cb_document, path, supported_extensions) == false) {
+      goto error_free;
+    }
   }
 
   girara_list_free(supported_extensions);
@@ -207,6 +213,43 @@ read_archive(cb_document_t* cb_document, const char* archive, girara_list_t* sup
   return true;
 }
 
+static bool
+read_dir(cb_document_t* cb_document, const char* directory, girara_list_t* supported_extensions)
+{
+  GDir* dir = g_dir_open(directory, 0, NULL);
+  const char* entrypath = NULL;
+  while ((entrypath = g_dir_read_name(dir))) {
+    char* fullpath = g_strdup_printf("%s/%s", directory, entrypath);
+    char* extension = get_extension(fullpath);
+    if (extension == NULL) {
+        continue;
+    }
+
+    GIRARA_LIST_FOREACH(supported_extensions, char*, iter, ext)
+      if (g_strcmp0(ext, extension) == 0) {
+        cb_document_page_meta_t* meta = g_malloc(sizeof(cb_document_page_meta_t));
+        meta->file = g_strdup(fullpath);
+        g_free(fullpath);
+        GdkPixbuf* data = gdk_pixbuf_new_from_file(meta->file, NULL);
+        meta->width = gdk_pixbuf_get_width(data);
+        meta->height = gdk_pixbuf_get_height(data);
+
+        if (meta->width > 0 && meta->height > 0) {
+          girara_list_append(cb_document->pages, meta);
+        } else {
+          cb_document_page_meta_free(meta);
+        }
+
+        break;
+      }
+    GIRARA_LIST_FOREACH_END(supported_extensions, char*, iter, ext);
+
+    g_free(extension);
+  }
+  g_dir_close(dir);
+  return true;
+}
+
 static int
 compare_pages(const cb_document_page_meta_t* page1, const cb_document_page_meta_t* page2)
 {
@@ -227,3 +270,4 @@ get_extension(const char* path)
 
   return g_ascii_strdown(res + 1, -1);
 }
+
